##########Load family data.
setwd("path")
data<-read.table("data.csv", header=T, sep=";",stringsAsFactor=F)

#We consider ID, FID, PID, MID, Sex, FID as columns for ID, Family ID, Father ID, Mother ID, Sex (coded as 0=Males,1=Females).

#Order data by Family ID, it is necessary for the next functions to work properly
data<-data[order(data$FID),]

##########Create Kinship matrix 

library(kinship2)
kped<-with(data, kinship2::pedigree(id = data$ID, dadid=data$PID, momid=data$MID, sex=data$Sex, famid=data$FID, missid="0"))
kmat <- 2*as.matrix(kinship(kped))

##########Create sharedenvironment matrix

#Use the function "sharedenvironmentmatrix" to create a correlation matrix between an individual 
#and his/her: father, mother, sibling, grandparents, uncles, cousins, spouses. 
#The function works as: sharedenvironmentmatrix(data, father, mother, sibling, grandparents, uncles, cousins, spouses,n.cores)
# Put 1 in the respective location to create the correlation.
source("sharedenvironmentmatrix.R")

# data2 is required for shared environment matrix creation
# Replace columns names as explained above. 
data2 <- data[, c("FID","ID","PID","MID","Sex")]

#In this example we model the shared environment effects considering the siblings (third position)
h<-sharedenvironmentmatrix(data2,0,0,1,0,0,0,0, n.cores=1)
hmat<-as.matrix(h)

##########We order the matrices as for data IDs order
hmat<-hmat[as.character(data$ID),as.character(data$ID)]
kmat<-kmat[as.character(data$ID),as.character(data$ID)]

##########Create Identity matrix
I<-diag(nrow(data))

########## Variables creation necessary for STAN code.
# You can use this function.
source("STANdatapreparation.R")
#STANdatapreparation(data, FamilyID,prevalence,number_covariates, covariates, Proband, Disease)

#Replace the arguments with your data

# data = data
# FamilyID = name of the column for Family ID
# prevalence = disease prevalence
# number_covariates = number of covariates included in the model
# covariates = vector with the name of the covariates 
# Proband = Name of the column of the proband
# Disease = Name of the column of the disease
  
#Example
STANdatapreparation(data=data, FamilyID="FamilyID", prevalence=0.01, number_covariates=2, covariates=c("COV1","COV2"), Proband="Proband", Disease="Disease")

#####Model implementation using STAN. cmdstanr R package needs to be installed.

library(cmdstanr)

####IF YOU HAVE TWO COVARIATES IN THE MODEL INCLUDED IN THE MODEL. IF YOU HAVE MORE THAN TWO COVARIATES YOU HAVE TO UPDATE THE STAN CODE (LOOK AT THE BOTTOM OF STAN CODE FOR GUIDELINES).

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_TwoCovariates.stan", cpp_options = list(stan_threads = TRUE))

#Fit the model with the variables previously created. You don't need to rename variables if you did not change any name. 
fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat,
  sharedenvironment_matrix_sibs=hmat,
  identity_matrix=I,
  threshold=threshold,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  X=X,
  SD=SD,
  XP=XP,
  YP=YP,
  NCOV=NCOV,
  fam=fam,
  grainsize=1),
  show_messages = FALSE,
  chains =4, #You can change the number of chains
  parallel_chains = 1, #Parallelize the chains. Max number = number of chains set.
  threads_per_chain = 1, #Parallelize the families. Max number = number of families in your dataset.
  refresh = 100,
  iter_warmup = 2000, iter_sampling = 3000) #You can change the number of iterations

#Print posterior distribution results
fit$print(variables=c("heritability_marginal",
                      "sharedenvironment_sibs_marginal",
                      "individualenvironment_marginal",
                      "beta_COV1",
                      "beta_COV2",
                      "tauCOV1COV2_marginal",
                      "tau_COV1_marginal",
                      "tau_COV2_marginal",
                      "doublecovariance_COV1COV2_marginal"),max_rows=100)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)

####IF YOU HAVE ONE COVARIATE INCLUDED IN THE MODEL
#Example
STANdatapreparation(data=data, FamilyID="FamilyID", prevalence=0.01, number_covariates=1, covariates=c("COV"), Proband="Proband", Disease="Disease")

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_OneCovariate.stan", cpp_options = list(stan_threads = TRUE))

fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat,
  sharedenvironment_matrix_sibs=hmat,
  identity_matrix=I,
  threshold=threshold,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  X=X,
  SD=SD,
  XP=XP,
  YP=YP,
  NCOV=NCOV,
  fam=fam,
  grainsize=1),
  show_messages = FALSE,
  chains =4, #You can change the number of chains
  parallel_chains = 1, #Parallelize the chains. Max number = number of chains set.
  threads_per_chain = 1, #Parallelize the families. Max number = number of families in your dataset.
  refresh = 100,
  iter_warmup = 2000, iter_sampling = 3000) #You can change the number of iterations

#Print posterior distribution results
fit$print(variables=c("heritability",
                      "sharedenvironment_sibs",
                      "individualenvironment",
                      "beta_COV",
                      "tauCOV_marginal"),max_rows=100)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)


####IF YOU DON'T HAVE ANY COVARIATE INCLUDED IN THE MODEL
#Example
STANdatapreparation(data=data, FamilyID="FamilyID", prevalence=0.01, number_covariates=0, covariates=NULL, Proband="Proband", Disease="Disease")

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_NoCovariates.stan", cpp_options = list(stan_threads = TRUE))

fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat,
  sharedenvironment_matrix_sibs=hmat,
  identity_matrix=I,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  fam=fam,
  X=X,
  grainsize=1),
  show_messages = FALSE,
  chains =4, #You can change the number of chains
  parallel_chains = 1, #Parallelize the chains. Max number = number of chains set.
  threads_per_chain = 1, #Parallelize the families. Max number = number of families in your dataset.
  refresh = 100,
  iter_warmup = 2000, iter_sampling = 3000) #You can change the number of iterations

#Print posterior distribution results
fit$print(variables=c("heritability",
                      "sharedenvironment_sibs",
                      "individualenvironment"),max_rows=100)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)


