##########Load data
setwd("path")
data<-read.table("Dataset_Sardinia.csv", header=T, sep=";",stringsAsFactor=F)
data<-data[order(data$FID),]

##########Create Kinship matrix

library(kinship2)
kped<-with(data, kinship2::pedigree(id = data$ID, dadid=data$PID, momid=data$MID, sex=data$Sex, famid=data$FID, missid="0"))
kmat <- 2*as.matrix(kinship(kped))

##########Create sharedenvironment matrices
source("sharedenvironmentmatrix.R")
#sharedenvironmentmatrix(data2, father, mother, sibling, grandparents, uncles, cousins, spouses,n.cores)

data2 <- data[, c("FID","ID","PID","MID","Sex")]

#Siblings shared environment
h<-sharedenvironmentmatrix(data2,0,0,1,0,0,0,0, n.cores=1)
hmat<-as.matrix(h)

#Mother-offspring shared environment
h<-sharedenvironmentmatrix(data2,0,1,0,0,0,0,0, n.cores=1)
hmat2<-as.matrix(h)

#Father-offspring shared environment
h<-sharedenvironmentmatrix(data2,1,0,0,0,0,0,0, n.cores=1)
hmat3<-as.matrix(h)

#Spouses shared environment
h<-sharedenvironmentmatrix(data2,0,0,0,0,0,0,1, n.cores=1)
hmat4<-as.matrix(h)

##########Retain individuals for the analysis: probands' relatives up to to the third generation and controls >= 20 years old  
data<-data[which(data$Retain==1),]

hmat<-hmat[as.character(data$ID),as.character(data$ID)]
hmat2<-hmat2[as.character(data$ID),as.character(data$ID)]
hmat3<-hmat3[as.character(data$ID),as.character(data$ID)]
hmat4<-hmat4[as.character(data$ID),as.character(data$ID)]
kmat<-kmat[as.character(data$ID),as.character(data$ID)]

##########Identity matrix
I<-diag(nrow(data))

########## Variables creation necessary for STAN code
#Number of Subjects
N<-nrow(data)

#Number of families
nfam<-length(unique(data$FID))

#Sequence number of families
fam<-seq(1,nfam)

#Save in "ni" the number of individuals within families and "firstfam" the row number for the first individual within each family
families<-data$FID
families<-families[!duplicated(families)]
i<-1
ni<-NULL
firstfam=NULL
while(i<nfam+1){
  famm <- which(data$FID==families[i])
  ni[i] <- length(famm)
  firstfam[i]<-famm[1]
  i<-i+1}

# Fix MS prevalence
prev<-0.00157
#Fix liability threshold
threshold <- qnorm(prev,0,1,lower.tail=F)

#Create indexes for cases and controls, and lower/upper bounds for the multivariate normal distribution
k <- which(data$MS==1) # cases
kk <- which(data$MS==0) # controls
lb <- rep(-Inf,nrow(data))
lb_ind <- rep(0,nrow(data))
lb[k] <- threshold
lb_ind[k] <- 1
ub <- rep(Inf,nrow(data))
ub_ind <- rep(0,nrow(data))
ub[kk] <- threshold
ub_ind[kk] <- 1

##########Create dummy variable for Year of birth Group
data$YearofBirthGroup_dummy<-ifelse(data$YearofBirthGroup==">=1946",1,0)

#Build matrix with standardized covariates
X<-as.matrix(cbind(scale(data$Sex),scale(data$YearofBirthGroup_dummy)))

#Covariates standard deviations
SD<-c(sd(data$Sex), sd(data$YearofBirthGroup_dummy))

#Number of covariates
NCOV<-ncol(X)

#Assign covariate for a single proband as the average between probands within the same family
data$SEX2<-X[,1]
data$YearofBirthGroup_dummy2<-X[,2]
prob<-data[which(data$Proband==1),]
prob2<- aggregate(prob$SEX2, list(prob$FID), FUN=mean) 
XP<-as.matrix(prob2$x[which(prob2$Group.1%in%data$FID)])
prob2<- aggregate(prob$YearofBirthGroup_dummy2, list(prob$FID), FUN=mean) 
XP<-cbind(XP,as.matrix(prob2$x[which(prob2$Group.1%in%data$FID)]))

#Probands MS status (all cases)
YP<-rep(1,nfam) 

#####Model implementation using STAN

library(cmdstanr)

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_Sardinians_Overall.stan", cpp_options = list(stan_threads = TRUE))

#Fit the model with the variables previously created
fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat,
  sharedenvironment_matrix_sibs=hmat,
  sharedenvironment_matrix_mother=hmat2,
  sharedenvironment_matrix_father=hmat3,
  sharedenvironment_matrix_spouses=hmat4,
  identity_matrix=I,
  threshold=threshold,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  X=X,
  SD=SD,
  XP=XP,
  YP=YP,
  NCOV=NCOV,
  fam=fam,
  grainsize=1),
  show_messages = FALSE,
  chains =1,
  parallel_chains = 1,
  threads_per_chain = 1,
  refresh = 100,
  iter_warmup = 1000, iter_sampling = 1000)

#Print posterior distribution results
fit$print(variables=c("heritability_marginal",
                      "sharedenvironment_sibs_marginal",
                      "sharedenvironment_mother_marginal",
                      "sharedenvironment_father_marginal",
                      "sharedenvironment_spouses_marginal",
                      "sharedenvironment_marginal",
                      "individualenvironment_marginal",
                      "tauSexYearBirthGroup_marginal",
                      "tau_YearBirthGroup_marginal",
                      "tau_Sex_marginal",
                      "beta_Sex",
                      "beta_YearBirthGroup"),max_rows=100
)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)


#########Stratified analysis
#<1946 group 
data2<-data[which(data$YearofBirthGroup=="<1946" & data$Retain==1),]

hmat_2<-hmat[as.character(data2$ID),as.character(data2$ID)]
hmat2_2<-hmat2[as.character(data2$ID),as.character(data2$ID)]
hmat3_2<-hmat3[as.character(data2$ID),as.character(data2$ID)]
hmat4_2<-hmat4[as.character(data2$ID),as.character(data2$ID)]
kmat_2<-kmat[as.character(data2$ID),as.character(data2$ID)]

##########Identity matrix
I<-diag(nrow(data2))

########## Variables creation necessary for STAN code
#Number of Subjects
N<-nrow(data2)

#Number of families
nfam<-length(unique(data2$FID))

#Sequence number of families
fam<-seq(1,nfam)

#Save in "ni" the number of individuals within families and "firstfam" the row number for the first individual within each family
families<-data2$FID
families<-families[!duplicated(families)]
i<-1
ni<-NULL
firstfam=NULL
while(i<nfam+1){
  famm <- which(data2$FID==families[i])
  ni[i] <- length(famm)
  firstfam[i]<-famm[1]
  i<-i+1}

# Fix MS prevalence
prev<-0.00103
#Fix liability threshold
threshold <- qnorm(prev,0,1,lower.tail=F)

#Create indexes for cases and controls, and lower/upper bounds for the multivariate normal distribution
k <- which(data2$MS==1) # cases
kk <- which(data2$MS==0) # controls
lb <- rep(-Inf,nrow(data2))
lb_ind <- rep(0,nrow(data2))
lb[k] <- threshold
lb_ind[k] <- 1
ub <- rep(Inf,nrow(data2))
ub_ind <- rep(0,nrow(data2))
ub[kk] <- threshold
ub_ind[kk] <- 1

#Build matrix with standardized covariates
X<-as.matrix(cbind(scale(data2$Sex)))

#Covariate standard deviations
SD<-c(sd(data2$Sex))

#Number of covariates
NCOV<-ncol(X)

#Use the probands' Sex values as calculated from the overall analysis.
XP_2<-XP[,1]

#Probands MS status (all cases)
YP<-rep(1,nfam) 

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_Sardinians_Before1946.stan", cpp_options = list(stan_threads = TRUE))

#Fit the model with the variables previously created
fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat_2,
  sharedenvironment_matrix_sibs=hmat_2,
  sharedenvironment_matrix_mother=hmat2_2,
  sharedenvironment_matrix_father=hmat3_2,
  sharedenvironment_matrix_spouses=hmat4_2,
  identity_matrix=I,
  threshold=threshold,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  X=X,
  SD=SD,
  XP=XP_2,
  YP=YP,
  NCOV=NCOV,
  fam=fam,
  grainsize=1),
  show_messages = FALSE,
  chains =4,
  parallel_chains = 1,
  threads_per_chain = 1,
  refresh = 100,
  iter_warmup = 2000, iter_sampling = 3000)

#Print posterior distribution results
fit$print(variables=c("heritability_marginal",
                      "sharedenvironment_sibs_marginal",
                      "sharedenvironment_mother_marginal",
                      "sharedenvironment_father_marginal",
                      "sharedenvironment_spouses_marginal",
                      "sharedenvironment_marginal",
                      "individualenvironment_marginal",
                      "tauSex_marginal",
                      "beta_Sex"),max_rows=100
)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)

round(summary(as.vector(fit$draws(variables="heritability_marginal"))),3)
round(sd(as.vector(fit$draws(variables="beta_Sex"))),3)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)


###>=1946 group 
data3<-data[which(data$YearofBirthGroup==">=1946 group" & data$Retain==1),]

hmat_3<-hmat[as.character(data3$ID),as.character(data3$ID)]
hmat2_3<-hmat2[as.character(data3$ID),as.character(data3$ID)]
hmat3_3<-hmat3[as.character(data3$ID),as.character(data3$ID)]
hmat4_3<-hmat4[as.character(data3$ID),as.character(data3$ID)]
kmat_3<-kmat[as.character(data3$ID),as.character(data3$ID)]

##########Identity matrix
I<-diag(nrow(data3))

########## Variables creation necessary for STAN code
#Number of Subjects
N<-nrow(data3)

#Number of families
nfam<-length(unique(data3$FID))

#Sequence number of families
fam<-seq(1,nfam)

#Save in "ni" the number of individuals within families and "firstfam" the row number for the first individual within each family
families<-data3$FID
families<-families[!duplicated(families)]
i<-1
ni<-NULL
firstfam=NULL
while(i<nfam+1){
  famm <- which(data3$FID==families[i])
  ni[i] <- length(famm)
  firstfam[i]<-famm[1]
  i<-i+1}

# Fix MS prevalence
prev<-0.00176
#Fix liability threshold
threshold <- qnorm(prev,0,1,lower.tail=F)

#Create indexes for cases and controls, and lower/upper bounds for the multivariate normal distribution
k <- which(data3$MS==1) # cases
kk <- which(data3$MS==0) # controls
lb <- rep(-Inf,nrow(data3))
lb_ind <- rep(0,nrow(data3))
lb[k] <- threshold
lb_ind[k] <- 1
ub <- rep(Inf,nrow(data3))
ub_ind <- rep(0,nrow(data3))
ub[kk] <- threshold
ub_ind[kk] <- 1


#Build matrix with standardized covariates
X<-as.matrix(cbind(scale(data3$Sex),scale(data3$YearofBirth)))

#Covariates standard deviations
SD<-c(sd(data3$Sex),sd(data3$YearofBirth))

#Number of covariates
NCOV<-ncol(X)

#Assign covariate for a single proband as the average between probands within the same family
data$SEX2<-X[,1]
data$YearofBirth2<-X[,2]
prob<-data[which(data$Proband==1),]
prob2<- aggregate(prob$SEX2, list(prob$FID), FUN=mean) 
XP<-as.matrix(prob2$x[which(prob2$Group.1%in%data$FID)])
prob2<- aggregate(prob$YearofBirth2, list(prob$FID), FUN=mean) 
XP<-cbind(XP,as.matrix(prob2$x[which(prob2$Group.1%in%data3$FID)]))

#Probands MS status (all cases)
YP<-rep(1,nfam) 

#Load STAN Code, stan_threads = TRUE to use parallelization
model <- cmdstan_model("Stan_code_ACEmodel_Sardinians_On_after1946.stan", cpp_options = list(stan_threads = TRUE))

#Fit the model with the variables previously created
fit <- model$sample(data=list(
  N=N,
  NFAM=nfam,
  kinship_matrix=kmat_3,
  sharedenvironment_matrix_sibs=hmat_3,
  sharedenvironment_matrix_mother=hmat2_3,
  sharedenvironment_matrix_father=hmat3_3,
  sharedenvironment_matrix_spouses=hmat4_3,
  identity_matrix=I,
  threshold=threshold,
  lb=lb,
  ub=ub,
  lb_ind=lb_ind,
  ub_ind=ub_ind,
  ni=ni,
  firstfam=firstfam,
  X=X,
  SD=SD,
  XP=XP,
  YP=YP,
  NCOV=NCOV,
  fam=fam,
  grainsize=1),
  show_messages = FALSE,
  chains =4,
  parallel_chains = 1,
  threads_per_chain = 1,
  refresh = 100,
  iter_warmup = 2000, iter_sampling = 3000)

#Print posterior distribution results
fit$print(variables=c("heritability_marginal",
                      "sharedenvironment_sibs_marginal",
                      "sharedenvironment_mother_marginal",
                      "sharedenvironment_father_marginal",
                      "sharedenvironment_spouses_marginal",
                      "sharedenvironment_marginal",
                      "individualenvironment_marginal",
                      "tauSexYearBirth_marginal",
                      "tau_YearBirth_marginal",
                      "tau_Sex_marginal",
                      "beta_Sex",
                      "beta_YearBirth"),max_rows=100)

# To obtain 95% HPD credibility intervals for the paramter of interest
library(HDInterval)
round(hdi(as.vector(fit$draws(variables="heritability_marginal"))),3)

